/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef BAIDUNLPENGINE_P_H
#define BAIDUNLPENGINE_P_H

#include <functional>
#include <jsoncpp/json/json.h>
#include <kylin-ai/ai-engine/large-model/cloudnlpengine.h>
#include <kylin-ai/ai-engine/large-model/error.h>
#include "cpr/cpr.h"

class BaiduNlpEnginePrivate
    : public ai_engine::lm::nlp::AbstractCloudNlpEngine {
public:
    BaiduNlpEnginePrivate();
    ~BaiduNlpEnginePrivate();
    std::string engineName() override;

    void setConfig(const std::string &config) override;
    void setChatResultCallback(
        ai_engine::lm::nlp::ChatResultCallback callback) override;

    void setContextSize(int size) override;
    void clearContext() override;

    bool chat(const std::string &message, ai_engine::lm::EngineError &error) override;
    void stopChat() override;

    bool initChatModule(ai_engine::lm::EngineError &error) override;
    bool destroyChatModule(ai_engine::lm::EngineError &error) override;

private:
    std::string apiKey_;
    std::string secretKey_;
    std::string accessToken_;
    bool needRefreshAccessToken_{false};
    std::string &getAccessToken_();
    bool generateAccessToken_();

    ai_engine::lm::nlp::ChatResultCallback chatResultCallback_;

    Json::Value context_;
    void addContext_(const std::string &role, const std::string &message);
    void removeLastContext_();
    std::vector<Json::Value> streamChatData_;
    std::string tempStreamChatData_;

    friend bool writeChatData(std::string data, intptr_t userData);

    bool processChatResponse_(cpr::Response &response, ai_engine::lm::EngineError &error);

    ai_engine::lm::EngineError currentError_;

    bool isStopped_{false};

    static std::string systemRole_;
};

#endif // BAIDUNLPENGINE_P_H
